@@ -17,48 +17,31 @@ import collections
 import contextlib
 import copy
 import datetime
+import functools
 import json
 import operator
 
 from automaton import exceptions as automaton_errors
-from ironicclient import exceptions
-from oslo_concurrency import lockutils
+from openstack import exceptions as os_exc
 from oslo_config import cfg
 from oslo_db.sqlalchemy import utils as db_utils
 from oslo_utils import excutils
 from oslo_utils import reflection
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
 from sqlalchemy.orm import exc as orm_errors
 
 from ironic_inspector.common.i18n import _
 from ironic_inspector.common import ironic as ir_utils
+from ironic_inspector.common import locking
 from ironic_inspector import db
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import utils
 
 
 CONF = cfg.CONF
-
-
 LOG = utils.getProcessingLogger(__name__)
-
-
 MACS_ATTRIBUTE = 'mac'
-_LOCK_TEMPLATE = 'node-%s'
-_SEMAPHORES = lockutils.Semaphores()
-
-
-def _get_lock(uuid):
-    """Get lock object for a given node UUID."""
-    return lockutils.internal_lock(_LOCK_TEMPLATE % uuid,
-                                   semaphores=_SEMAPHORES)
-
-
-def _get_lock_ctx(uuid):
-    """Get context manager yielding a lock object for a given node UUID."""
-    return lockutils.lock(_LOCK_TEMPLATE % uuid, semaphores=_SEMAPHORES)
 
 
 class NodeInfo(object):
@@ -71,7 +54,7 @@ class NodeInfo(object):
 
     def __init__(self, uuid, version_id=None, state=None, started_at=None,
                  finished_at=None, error=None, node=None, ports=None,
-                 ironic=None, lock=None, manage_boot=True):
+                 ironic=None, manage_boot=True):
         self.uuid = uuid
         self.started_at = started_at
         self.finished_at = finished_at
@@ -89,13 +72,12 @@ class NodeInfo(object):
         # equivalent to True actually.
         self._manage_boot = manage_boot if manage_boot is not None else True
         # This is a lock on a node UUID, not on a NodeInfo object
-        self._lock = lock if lock is not None else _get_lock(uuid)
+        self._lock = locking.get_lock(uuid)
         # Whether lock was acquired using this NodeInfo object
-        self._locked = lock is not None
         self._fsm = None
 
     def __del__(self):
-        if self._locked:
+        if self._lock.is_locked():
             LOG.warning('BUG: node lock was not released by the moment '
                         'node info object is deleted')
             self._lock.release()
@@ -117,12 +99,13 @@ class NodeInfo(object):
                          return immediately.
         :returns: boolean value, whether lock was acquired successfully
         """
-        if self._locked:
+        if self._lock.is_locked():
+            LOG.debug('Attempting to acquire lock already held',
+                      node_info=self)
             return True
 
         LOG.debug('Attempting to acquire lock', node_info=self)
         if self._lock.acquire(blocking):
-            self._locked = True
             LOG.debug('Successfully acquired lock', node_info=self)
             return True
         else:
@@ -134,10 +117,9 @@ class NodeInfo(object):
 
         Does nothing if lock was not acquired using this NodeInfo object.
         """
-        if self._locked:
+        if self._lock.is_locked():
             LOG.debug('Successfully released lock', node_info=self)
             self._lock.release()
-        self._locked = False
 
     @property
     def version_id(self):
@@ -319,12 +301,12 @@ class NodeInfo(object):
             self._attributes = None
 
     @classmethod
-    def from_row(cls, row, ironic=None, lock=None, node=None):
+    def from_row(cls, row, ironic=None, node=None):
         """Construct NodeInfo from a database row."""
         fields = {key: row[key]
                   for key in ('uuid', 'version_id', 'state', 'started_at',
                               'finished_at', 'error', 'manage_boot')}
-        return cls(ironic=ironic, lock=lock, node=node, **fields)
+        return cls(ironic=ironic, node=node, **fields)
 
     def invalidate_cache(self):
         """Clear all cached info, so that it's reloaded next time."""
@@ -359,17 +341,18 @@ class NodeInfo(object):
         for port in ports:
             mac = port
             extra = {}
-            pxe_enabled = True
+            is_pxe_enabled = True
             if isinstance(port, dict):
                 mac = port['mac']
                 client_id = port.get('client_id')
                 if client_id:
                     extra = {'client-id': client_id}
-                pxe_enabled = port.get('pxe', True)
+                if CONF.processing.update_pxe_enabled:
+                    is_pxe_enabled = port.get('pxe', True)
 
             if mac not in self.ports():
                 self._create_port(mac, ironic=ironic, extra=extra,
-                                  pxe_enabled=pxe_enabled)
+                                  is_pxe_enabled=is_pxe_enabled)
             else:
                 existing_macs.append(mac)
 
@@ -386,21 +369,21 @@ class NodeInfo(object):
         """
         if self._ports is None:
             ironic = ironic or self.ironic
-            port_list = ironic.node.list_ports(self.uuid, limit=0, detail=True)
+            port_list = ironic.ports(node=self.uuid, limit=None, details=True)
             self._ports = {p.address: p for p in port_list}
         return self._ports
 
     def _create_port(self, mac, ironic=None, **kwargs):
         ironic = ironic or self.ironic
         try:
-            port = ironic.port.create(
+            port = ironic.create_port(
                 node_uuid=self.uuid, address=mac, **kwargs)
             LOG.info('Port %(uuid)s was created successfully, MAC: %(mac)s,'
                      'attributes: %(attrs)s',
-                     {'uuid': port.uuid, 'mac': port.address,
+                     {'uuid': port.id, 'mac': port.address,
                       'attrs': kwargs},
                      node_info=self)
-        except exceptions.Conflict:
+        except os_exc.ConflictException:
             LOG.warning('Port %s already exists, skipping',
                         mac, node_info=self)
             # NOTE(dtantsur): we didn't get port object back, so we have to
@@ -417,7 +400,7 @@ class NodeInfo(object):
         :param patches: JSON patches to apply
         :param ironic: Ironic client to use instead of self.ironic
         :param kwargs: Arguments to pass to ironicclient.
-        :raises: ironicclient exceptions
+        :raises: openstacksdk exceptions
         """
         ironic = ironic or self.ironic
         # NOTE(aarefiev): support path w/o ahead forward slash
@@ -427,7 +410,7 @@ class NodeInfo(object):
                 patch['path'] = '/' + patch['path']
 
         LOG.debug('Updating node with patches %s', patches, node_info=self)
-        self._node = ironic.node.update(self.uuid, patches, **kwargs)
+        self._node = ironic.patch_node(self.uuid, patches, **kwargs)
 
     def patch_port(self, port, patches, ironic=None):
         """Apply JSON patches to a port.
@@ -438,13 +421,13 @@ class NodeInfo(object):
         """
         ironic = ironic or self.ironic
         ports = self.ports()
-        if isinstance(port, six.string_types):
+        if isinstance(port, str):
             port = ports[port]
 
         LOG.debug('Updating port %(mac)s with patches %(patches)s',
                   {'mac': port.address, 'patches': patches},
                   node_info=self)
-        new_port = ironic.port.update(port.uuid, patches)
+        new_port = ironic.patch_port(port.id, patches)
         ports[port.address] = new_port
 
     def update_properties(self, ironic=None, **props):
@@ -478,7 +461,7 @@ class NodeInfo(object):
         :param ironic: Ironic client to use instead of self.ironic
         """
         ironic = ironic or self.ironic
-        ironic.node.add_trait(self.uuid, trait)
+        ironic.add_node_trait(self.uuid, trait)
 
     def remove_trait(self, trait, ironic=None):
         """Remove a trait from the node.
@@ -488,8 +471,8 @@ class NodeInfo(object):
         """
         ironic = ironic or self.ironic
         try:
-            ironic.node.remove_trait(self.uuid, trait)
-        except exceptions.NotFound:
+            ironic.remove_node_trait(self.uuid, trait)
+        except os_exc.NotFoundException:
             LOG.debug('Trait %s is not set, cannot remove', trait,
                       node_info=self)
 
@@ -501,10 +484,10 @@ class NodeInfo(object):
         """
         ironic = ironic or self.ironic
         ports = self.ports()
-        if isinstance(port, six.string_types):
+        if isinstance(port, str):
             port = ports[port]
 
-        ironic.port.delete(port.uuid)
+        ironic.delete_port(port.id)
         del ports[port.address]
 
     def get_by_path(self, path):
@@ -565,7 +548,7 @@ def triggers_fsm_error_transition(errors=(Exception,),
                       error event.
     """
     def outer(func):
-        @six.wraps(func)
+        @functools.wraps(func)
         def inner(node_info, *args, **kwargs):
             ret = None
             try:
@@ -601,7 +584,7 @@ def fsm_event_before(event, strict=False):
     :param strict: make an invalid fsm event trigger an error event
     """
     def outer(func):
-        @six.wraps(func)
+        @functools.wraps(func)
         def inner(node_info, *args, **kwargs):
             LOG.debug('Processing event %(event)s before calling '
                       '%(func)s', {'event': event, 'func': func},
@@ -622,7 +605,7 @@ def fsm_event_after(event, strict=False):
     :param strict: make an invalid fsm event trigger an error event
     """
     def outer(func):
-        @six.wraps(func)
+        @functools.wraps(func)
         def inner(node_info, *args, **kwargs):
             ret = func(node_info, *args, **kwargs)
             LOG.debug('Processing event %(event)s after calling '
@@ -661,14 +644,14 @@ def release_lock(func):
     instance.
 
     """
-    @six.wraps(func)
+    @functools.wraps(func)
     def inner(node_info, *args, **kwargs):
         try:
             return func(node_info, *args, **kwargs)
         finally:
             # FIXME(milan) hacking the test cases to work
             # with release_lock.assert_called_once...
-            if node_info._locked:
+            if node_info._lock.is_locked():
                 node_info.release_lock()
     return inner
 
@@ -743,7 +726,7 @@ def delete_nodes_not_in_list(uuids):
     for uuid in inspector_uuids - uuids:
         LOG.warning('Node %s was deleted from Ironic, dropping from Ironic '
                     'Inspector database', uuid)
-        with _get_lock_ctx(uuid):
+        with locking.get_lock(uuid):
             _delete_node(uuid)
 
 
@@ -783,12 +766,11 @@ def _list_node_uuids():
     return {x.uuid for x in db.model_query(db.Node.uuid)}
 
 
-def get_node(node_id, ironic=None, locked=False):
+def get_node(node_id, ironic=None):
     """Get node from cache.
 
     :param node_id: node UUID or name.
     :param ironic: optional ironic client instance
-    :param locked: if True, get a lock on node before fetching its data
     :returns: structure NodeInfo.
     """
     if uuidutils.is_uuid_like(node_id):
@@ -796,24 +778,13 @@ def get_node(node_id, ironic=None, locked=False):
         uuid = node_id
     else:
         node = ir_utils.get_node(node_id, ironic=ironic)
-        uuid = node.uuid
+        uuid = node.id
 
-    if locked:
-        lock = _get_lock(uuid)
-        lock.acquire()
-    else:
-        lock = None
-
-    try:
-        row = db.model_query(db.Node).filter_by(uuid=uuid).first()
-        if row is None:
-            raise utils.Error(_('Could not find node %s in cache') % uuid,
-                              code=404)
-        return NodeInfo.from_row(row, ironic=ironic, lock=lock, node=node)
-    except Exception:
-        with excutils.save_and_reraise_exception():
-            if lock is not None:
-                lock.release()
+    row = db.model_query(db.Node).filter_by(uuid=uuid).first()
+    if row is None:
+        raise utils.Error(_('Could not find node %s in cache') % uuid,
+                          code=404)
+    return NodeInfo.from_row(row, ironic=ironic, node=node)
 
 
 def find_node(**attributes):
@@ -843,7 +814,7 @@ def find_node(**attributes):
         query = db.model_query(db.Attribute.node_uuid)
         pairs = [(db.Attribute.name == name) &
                  (db.Attribute.value == v) for v in value]
-        query = query.filter(six.moves.reduce(operator.or_, pairs))
+        query = query.filter(functools.reduce(operator.or_, pairs))
         found.update(row.node_uuid for row in query.distinct().all())
 
     if not found:
@@ -911,22 +882,28 @@ def clean_up():
         return []
 
     LOG.error('Introspection for nodes %s has timed out', uuids)
+    locked_uuids = []
     for u in uuids:
-        node_info = get_node(u, locked=True)
-        try:
-            if node_info.finished_at or node_info.started_at > threshold:
-                continue
-            if node_info.state != istate.States.waiting:
-                LOG.error('Something went wrong, timeout occurred '
-                          'while introspection in "%s" state',
-                          node_info.state,
-                          node_info=node_info)
-            node_info.finished(
-                istate.Events.timeout, error='Introspection timeout')
-        finally:
-            node_info.release_lock()
+        node_info = get_node(u)
+        if node_info.acquire_lock(blocking=False):
+            try:
+                if node_info.finished_at or node_info.started_at > threshold:
+                    continue
+                if node_info.state != istate.States.waiting:
+                    LOG.error('Something went wrong, timeout occurred '
+                              'while introspection in "%s" state',
+                              node_info.state,
+                              node_info=node_info)
+                node_info.finished(
+                    istate.Events.timeout, error='Introspection timeout')
+                locked_uuids.append(u)
+            finally:
+                node_info.release_lock()
+        else:
+            LOG.info('Failed to acquire lock when updating node state',
+                     node_info=node_info)
 
-    return uuids
+    return locked_uuids
 
 
 def create_node(driver, ironic=None, **attributes):
@@ -937,7 +914,7 @@ def create_node(driver, ironic=None, **attributes):
     * Sets node_info state to enrolling.
 
     :param driver: driver for Ironic node.
-    :param ironic: ronic client instance.
+    :param ironic: ironic client instance.
     :param attributes: dict, additional keyword arguments to pass
                              to the ironic client on node creation.
     :return: NodeInfo, or None in case error happened.
@@ -945,12 +922,49 @@ def create_node(driver, ironic=None, **attributes):
     if ironic is None:
         ironic = ir_utils.get_client()
     try:
-        node = ironic.node.create(driver=driver, **attributes)
-    except exceptions.InvalidAttribute as e:
+        node = ironic.create_node(driver=driver, **attributes)
+    except os_exc.SDKException as e:
         LOG.error('Failed to create new node: %s', e)
     else:
-        LOG.info('Node %s was created successfully', node.uuid)
-        return add_node(node.uuid, istate.States.enrolling, ironic=ironic)
+        LOG.info('Node %s was created successfully', node.id)
+        return add_node(node.id, istate.States.enrolling, ironic=ironic)
+
+
+def record_node(ironic=None, bmc_addresses=None, macs=None):
+    """Create a cache record for a known active node.
+
+    :param ironic: ironic client instance.
+    :param bmc_addresses: list of BMC addresses.
+    :param macs: list of MAC addresses.
+    :return: NodeInfo
+    """
+    if not bmc_addresses and not macs:
+        raise utils.NotFoundInCacheError(
+            _("Existing node cannot be found since neither MAC addresses "
+              "nor BMC addresses are present in the inventory"))
+
+    if ironic is None:
+        ironic = ir_utils.get_client()
+
+    node = ir_utils.lookup_node(macs=macs, bmc_addresses=bmc_addresses,
+                                ironic=ironic)
+    if not node:
+        bmc_addresses = ', '.join(bmc_addresses) if bmc_addresses else None
+        macs = ', '.join(macs) if macs else None
+        raise utils.NotFoundInCacheError(
+            _("Existing node was not found by MAC address(es) %(macs)s "
+              "and BMC address(es) %(addr)s") %
+            {'macs': macs, 'addr': bmc_addresses})
+
+    node = ironic.get_node(node, fields=['uuid', 'provision_state'])
+    # TODO(dtantsur): do we want to allow updates in all states?
+    if node.provision_state not in ir_utils.VALID_ACTIVE_STATES:
+        raise utils.Error(_("Node %(node)s is not active, its provision "
+                            "state is %(state)s") %
+                          {'node': node.id, 'state': node.provision_state})
+
+    return add_node(node.id, istate.States.waiting,
+                    manage_boot=False, mac=macs, bmc_address=bmc_addresses)
 
 
 def get_node_list(ironic=None, marker=None, limit=None):
