@@ -11,19 +11,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import builtins
+try:
+    import errno
+except ImportError:
+    import os.errno as errno
 import datetime
 import os
+from unittest import mock
 
 import fixtures
-from ironicclient import exc as ironic_exc
-import mock
+from openstack import exceptions as os_exc
 from oslo_config import cfg
-import six
 
 from ironic_inspector.common import ironic as ir_utils
 from ironic_inspector import node_cache
+from ironic_inspector.pxe_filter import base as pxe_filter
 from ironic_inspector.pxe_filter import dnsmasq
 from ironic_inspector.test import base as test_base
+from ironic_inspector import utils
 
 CONF = cfg.CONF
 
@@ -34,6 +40,15 @@ class DnsmasqTestBase(test_base.BaseTest):
         self.driver = dnsmasq.DnsmasqFilter()
 
 
+class TestConfiguration(DnsmasqTestBase):
+
+    def test_deny_unknown_macs_and_node_not_found_hook_bad(self):
+        CONF.set_override('deny_unknown_macs', True, 'pxe_filter')
+        CONF.set_override('node_not_found_hook', True, 'processing')
+        self.assertRaisesRegex(utils.Error, 'Configuration error:',
+                               self.driver.__init__)
+
+
 class TestShouldEnableUnknownHosts(DnsmasqTestBase):
     def setUp(self):
         super(TestShouldEnableUnknownHosts, self).setUp()
@@ -106,7 +121,7 @@ class TestExclusiveWriteOrPass(test_base.BaseTest):
     def setUp(self):
         super(TestExclusiveWriteOrPass, self).setUp()
         self.mock_open = self.useFixture(fixtures.MockPatchObject(
-            six.moves.builtins, 'open', new=mock.mock_open())).mock
+            builtins, 'open', new=mock.mock_open())).mock
         self.mock_fd = self.mock_open.return_value
         self.mock_fcntl = self.useFixture(fixtures.MockPatchObject(
             dnsmasq.fcntl, 'flock', autospec=True)).mock
@@ -122,7 +137,7 @@ class TestExclusiveWriteOrPass(test_base.BaseTest):
 
     def test_write(self):
         wrote = dnsmasq._exclusive_write_or_pass(self.path, self.buf)
-        self.assertEqual(True, wrote)
+        self.assertTrue(wrote)
         self.mock_open.assert_called_once_with(self.path, 'w', 1)
         self.mock_fcntl.assert_has_calls(
             [self.fcntl_lock_call, self.fcntl_unlock_call])
@@ -131,7 +146,7 @@ class TestExclusiveWriteOrPass(test_base.BaseTest):
 
     def test_write_would_block(self):
         err = IOError('Oops!')
-        err.errno = os.errno.EWOULDBLOCK
+        err.errno = errno.EWOULDBLOCK
         # lock/unlock paired calls
         self.mock_fcntl.side_effect = [
             # first try
@@ -140,7 +155,7 @@ class TestExclusiveWriteOrPass(test_base.BaseTest):
             None, None]
         wrote = dnsmasq._exclusive_write_or_pass(self.path, self.buf)
 
-        self.assertEqual(True, wrote)
+        self.assertTrue(wrote)
         self.mock_open.assert_called_once_with(self.path, 'w', 1)
         self.mock_fcntl.assert_has_calls(
             [self.fcntl_lock_call, self.fcntl_unlock_call],
@@ -156,11 +171,11 @@ class TestExclusiveWriteOrPass(test_base.BaseTest):
             'ironic_inspector.pxe_filter.dnsmasq._EXCLUSIVE_WRITE_ATTEMPTS',
             1))
         err = IOError('Oops!')
-        err.errno = os.errno.EWOULDBLOCK
+        err.errno = errno.EWOULDBLOCK
         self.mock_fcntl.side_effect = [err, None]
 
         wrote = dnsmasq._exclusive_write_or_pass(self.path, self.buf)
-        self.assertEqual(False, wrote)
+        self.assertFalse(wrote)
         self.mock_open.assert_called_once_with(self.path, 'w', 1)
         self.mock_fcntl.assert_has_calls(
             [self.fcntl_lock_call, self.fcntl_unlock_call])
@@ -180,7 +195,7 @@ class TestExclusiveWriteOrPass(test_base.BaseTest):
     def test_write_custom_ioerror(self):
 
         err = IOError('Oops!')
-        err.errno = os.errno.EBADF
+        err.errno = errno.EBADF
         self.mock_fcntl.side_effect = [err, None]
 
         self.assertRaisesRegex(
@@ -217,7 +232,7 @@ class TestMACHandlers(test_base.BaseTest):
         self.mock_introspection_active = self.useFixture(
             fixtures.MockPatchObject(node_cache, 'introspection_active')).mock
 
-    def test__whitelist_unknown_hosts(self):
+    def test__allowlist_unknown_hosts(self):
         self.mock_join.return_value = "%s/%s" % (self.dhcp_hostsdir,
                                                  dnsmasq._UNKNOWN_HOSTS_FILE)
         self.mock_introspection_active.return_value = True
@@ -227,12 +242,12 @@ class TestMACHandlers(test_base.BaseTest):
                                                dnsmasq._UNKNOWN_HOSTS_FILE)
         self.mock__exclusive_write_or_pass.assert_called_once_with(
             self.mock_join.return_value,
-            '%s' % dnsmasq._WHITELIST_UNKNOWN_HOSTS)
+            '%s' % dnsmasq._ALLOW_UNKNOWN_HOSTS)
         self.mock_log.debug.assert_called_once_with(
             'A %s record for all unknown hosts using wildcard mac '
-            'created', 'whitelist')
+            'created', 'allow')
 
-    def test__blacklist_unknown_hosts(self):
+    def test__denylist_unknown_hosts(self):
         self.mock_join.return_value = "%s/%s" % (self.dhcp_hostsdir,
                                                  dnsmasq._UNKNOWN_HOSTS_FILE)
         self.mock_introspection_active.return_value = False
@@ -242,14 +257,31 @@ class TestMACHandlers(test_base.BaseTest):
                                                dnsmasq._UNKNOWN_HOSTS_FILE)
         self.mock__exclusive_write_or_pass.assert_called_once_with(
             self.mock_join.return_value,
-            '%s' % dnsmasq._BLACKLIST_UNKNOWN_HOSTS)
+            '%s' % dnsmasq._DENY_UNKNOWN_HOSTS)
         self.mock_log.debug.assert_called_once_with(
             'A %s record for all unknown hosts using wildcard mac '
-            'created', 'blacklist')
+            'created', 'deny')
 
-    def test__configure_removedlist_whitelist(self):
+    def test__macs_unknown_hosts_deny_unknown(self):
+        self.mock_join.return_value = "%s/%s" % (self.dhcp_hostsdir,
+                                                 dnsmasq._UNKNOWN_HOSTS_FILE)
         self.mock_introspection_active.return_value = True
-        self.mock_stat.return_value.st_size = dnsmasq._MACBL_LEN
+        CONF.set_override('deny_unknown_macs', True, 'pxe_filter')
+
+        dnsmasq._configure_unknown_hosts()
+
+        self.mock_join.assert_called_once_with(self.dhcp_hostsdir,
+                                               dnsmasq._UNKNOWN_HOSTS_FILE)
+        self.mock__exclusive_write_or_pass.assert_called_once_with(
+            self.mock_join.return_value,
+            '%s' % dnsmasq._DENY_UNKNOWN_HOSTS)
+        self.mock_log.debug.assert_called_once_with(
+            'A %s record for all unknown hosts using wildcard mac '
+            'created', 'deny')
+
+    def test__configure_removedlist_allowlist(self):
+        self.mock_introspection_active.return_value = True
+        self.mock_stat.return_value.st_size = dnsmasq._MAC_DENY_LEN
 
         dnsmasq._configure_removedlist({self.mac})
 
@@ -257,9 +289,9 @@ class TestMACHandlers(test_base.BaseTest):
         self.mock__exclusive_write_or_pass.assert_called_once_with(
             self.mock_join.return_value, '%s\n' % self.mac)
 
-    def test__configure_removedlist_blacklist(self):
+    def test__configure_removedlist_denylist(self):
         self.mock_introspection_active.return_value = False
-        self.mock_stat.return_value.st_size = dnsmasq._MACWL_LEN
+        self.mock_stat.return_value.st_size = dnsmasq._MAC_ALLOW_LEN
 
         dnsmasq._configure_removedlist({self.mac})
 
@@ -267,56 +299,67 @@ class TestMACHandlers(test_base.BaseTest):
         self.mock__exclusive_write_or_pass.assert_called_once_with(
             self.mock_join.return_value, '%s,ignore\n' % self.mac)
 
-    def test__whitelist_mac(self):
-        dnsmasq._whitelist_mac(self.mac)
+    def test__configure_removedlist_denylist_deny_unknown(self):
+        self.mock_introspection_active.return_value = True
+        self.mock_stat.return_value.st_size = dnsmasq._MAC_ALLOW_LEN
+        CONF.set_override('deny_unknown_macs', True, 'pxe_filter')
+
+        dnsmasq._configure_removedlist({self.mac})
+
+        self.mock_join.assert_called_with(self.dhcp_hostsdir, self.mac)
+        self.mock__exclusive_write_or_pass.assert_called_once_with(
+            self.mock_join.return_value, '%s,ignore\n' % self.mac)
+
+    def test__allowlist_mac(self):
+        dnsmasq._add_mac_to_allowlist(self.mac)
 
         self.mock_join.assert_called_once_with(self.dhcp_hostsdir, self.mac)
         self.mock__exclusive_write_or_pass.assert_called_once_with(
             self.mock_join.return_value, '%s\n' % self.mac)
 
-    def test__blacklist_mac(self):
-        dnsmasq._blacklist_mac(self.mac)
+    def test__denylist_mac(self):
+        dnsmasq._add_mac_to_denylist(self.mac)
 
         self.mock_join.assert_called_once_with(self.dhcp_hostsdir, self.mac)
         self.mock__exclusive_write_or_pass.assert_called_once_with(
             self.mock_join.return_value, '%s,ignore\n' % self.mac)
 
-    def test__get_blacklist(self):
+    def test__get_denylist(self):
         self.mock_listdir.return_value = [self.mac]
         self.mock_stat.return_value.st_size = len('%s,ignore\n' % self.mac)
-        blacklist, whitelist = dnsmasq._get_black_white_lists()
+        denylist, allowlist = dnsmasq._get_deny_allow_lists()
 
-        self.assertEqual({self.mac}, blacklist)
+        self.assertEqual({self.mac}, denylist)
         self.mock_listdir.assert_called_once_with(self.dhcp_hostsdir)
         self.mock_join.assert_called_with(self.dhcp_hostsdir, self.mac)
         self.mock_stat.assert_called_with(self.mock_join.return_value)
 
-    def test__get_whitelist(self):
+    def test__get_allowlist(self):
         self.mock_listdir.return_value = [self.mac]
         self.mock_stat.return_value.st_size = len('%s\n' % self.mac)
-        blacklist, whitelist = dnsmasq._get_black_white_lists()
+        denylist, allowlist = dnsmasq._get_deny_allow_lists()
 
-        self.assertEqual({self.mac}, whitelist)
+        self.assertEqual({self.mac}, allowlist)
         self.mock_listdir.assert_called_once_with(self.dhcp_hostsdir)
         self.mock_join.assert_called_with(self.dhcp_hostsdir, self.mac)
         self.mock_stat.assert_called_with(self.mock_join.return_value)
 
-    def test__get_no_blacklist(self):
+    def test__get_no_denylist(self):
         self.mock_listdir.return_value = [self.mac]
         self.mock_stat.return_value.st_size = len('%s\n' % self.mac)
-        blacklist, whitelist = dnsmasq._get_black_white_lists()
+        denylist, allowlist = dnsmasq._get_deny_allow_lists()
 
-        self.assertEqual(set(), blacklist)
+        self.assertEqual(set(), denylist)
         self.mock_listdir.assert_called_once_with(self.dhcp_hostsdir)
         self.mock_join.assert_called_with(self.dhcp_hostsdir, self.mac)
         self.mock_stat.assert_called_with(self.mock_join.return_value)
 
-    def test__get_no_whitelist(self):
+    def test__get_no_allowlist(self):
         self.mock_listdir.return_value = [self.mac]
         self.mock_stat.return_value.st_size = len('%s,ignore\n' % self.mac)
-        blacklist, whitelist = dnsmasq._get_black_white_lists()
+        denylist, allowlist = dnsmasq._get_deny_allow_lists()
 
-        self.assertEqual(set(), whitelist)
+        self.assertEqual(set(), allowlist)
         self.mock_listdir.assert_called_once_with(self.dhcp_hostsdir)
         self.mock_join.assert_called_with(self.dhcp_hostsdir, self.mac)
         self.mock_stat.assert_called_with(self.mock_join.return_value)
@@ -344,12 +387,12 @@ class TestMACHandlers(test_base.BaseTest):
 class TestSync(DnsmasqTestBase):
     def setUp(self):
         super(TestSync, self).setUp()
-        self.mock__get_black_white_lists = self.useFixture(
-            fixtures.MockPatchObject(dnsmasq, '_get_black_white_lists')).mock
-        self.mock__whitelist_mac = self.useFixture(
-            fixtures.MockPatchObject(dnsmasq, '_whitelist_mac')).mock
-        self.mock__blacklist_mac = self.useFixture(
-            fixtures.MockPatchObject(dnsmasq, '_blacklist_mac')).mock
+        self.mock__get_deny_allow_lists = self.useFixture(
+            fixtures.MockPatchObject(dnsmasq, '_get_deny_allow_lists')).mock
+        self.mock__add_mac_to_allowlist = self.useFixture(
+            fixtures.MockPatchObject(dnsmasq, '_add_mac_to_allowlist')).mock
+        self.mock__add_mac_to_denylist = self.useFixture(
+            fixtures.MockPatchObject(dnsmasq, '_add_mac_to_denylist')).mock
         self.mock__configure_unknown_hosts = self.useFixture(
             fixtures.MockPatchObject(dnsmasq, '_configure_unknown_hosts')).mock
         self.mock__configure_removedlist = self.useFixture(
@@ -368,15 +411,18 @@ class TestSync(DnsmasqTestBase):
         get_client_mock = self.useFixture(
             fixtures.MockPatchObject(ir_utils, 'get_client')).mock
         get_client_mock.return_value = self.mock_ironic
+        self.mock_get_active_macs = self.useFixture(
+            fixtures.MockPatchObject(pxe_filter, 'get_active_macs')).mock
+        self.mock_get_active_macs.return_value = set()
         self.mock_active_macs = self.useFixture(
             fixtures.MockPatchObject(node_cache, 'active_macs')).mock
         self.ironic_macs = {'new_mac', 'active_mac'}
         self.active_macs = {'active_mac'}
-        self.blacklist = {'gone_mac', 'active_mac'}
-        self.whitelist = {}
-        self.mock__get_black_white_lists.return_value = (self.blacklist,
-                                                         self.whitelist)
-        self.mock_ironic.port.list.return_value = [
+        self.denylist = {'gone_mac', 'active_mac'}
+        self.allowlist = {}
+        self.mock__get_deny_allow_lists.return_value = (self.denylist,
+                                                        self.allowlist)
+        self.mock_ironic.ports.return_value = [
             mock.Mock(address=address) for address in self.ironic_macs]
         self.mock_active_macs.return_value = self.active_macs
         self.mock_should_enable_unknown_hosts = self.useFixture(
@@ -397,15 +443,16 @@ class TestSync(DnsmasqTestBase):
         self.mock__configure_unknown_hosts.assert_called_once_with()
 
     def test__sync(self):
+        self.mock_get_active_macs.return_value = {'active_mac'}
         self.driver._sync(self.mock_ironic)
 
-        self.mock__whitelist_mac.assert_called_once_with('active_mac')
-        self.mock__blacklist_mac.assert_called_once_with('new_mac')
+        self.mock_get_active_macs.assert_called_once_with(self.mock_ironic)
+        self.mock__add_mac_to_allowlist.assert_called_once_with('active_mac')
+        self.mock__add_mac_to_denylist.assert_called_once_with('new_mac')
 
-        self.mock_ironic.port.list.assert_called_once_with(limit=0,
-                                                           fields=['address'])
-        self.mock_active_macs.assert_called_once_with()
-        self.mock__get_black_white_lists.assert_called_once_with()
+        self.mock_ironic.ports.assert_called_once_with(
+            fields=['address', 'extra'], limit=None)
+        self.mock__get_deny_allow_lists.assert_called_once_with()
         self.mock__configure_unknown_hosts.assert_called_once_with()
         self.mock__configure_removedlist.assert_called_once_with({'gone_mac'})
         self.mock_log.debug.assert_has_calls([
@@ -416,19 +463,20 @@ class TestSync(DnsmasqTestBase):
 
     @mock.patch('time.sleep', lambda _x: None)
     def test__sync_with_port_list_retries(self):
-        self.mock_ironic.port.list.side_effect = [
-            ironic_exc.ConnectionRefused('boom'),
+        self.mock_ironic.ports.side_effect = [
+            os_exc.SDKException('boom'),
             [mock.Mock(address=address) for address in self.ironic_macs]
         ]
+        self.mock_get_active_macs.return_value = {'active_mac'}
         self.driver._sync(self.mock_ironic)
 
-        self.mock__whitelist_mac.assert_called_once_with('active_mac')
-        self.mock__blacklist_mac.assert_called_once_with('new_mac')
+        self.mock__add_mac_to_allowlist.assert_called_once_with('active_mac')
+        self.mock__add_mac_to_denylist.assert_called_once_with('new_mac')
 
-        self.mock_ironic.port.list.assert_called_with(limit=0,
-                                                      fields=['address'])
-        self.mock_active_macs.assert_called_once_with()
-        self.mock__get_black_white_lists.assert_called_once_with()
+        self.mock_ironic.ports.assert_called_with(fields=['address', 'extra'],
+                                                  limit=None)
+        self.mock_get_active_macs.assert_called_once_with(self.mock_ironic)
+        self.mock__get_deny_allow_lists.assert_called_once_with()
         self.mock__configure_removedlist.assert_called_once_with({'gone_mac'})
         self.mock_log.debug.assert_has_calls([
             mock.call('Syncing the driver'),
