@@ -13,15 +13,19 @@
 
 import datetime
 import json
+import os
+import tempfile
 import unittest
+from unittest import mock
 
 import fixtures
-import mock
 import oslo_messaging as messaging
 from oslo_utils import uuidutils
 
+from ironic_inspector.common import coordination
 from ironic_inspector.common import ironic as ir_utils
 from ironic_inspector.common import rpc
+from ironic_inspector.common import swift
 import ironic_inspector.conf
 from ironic_inspector.conf import opts as conf_opts
 from ironic_inspector import introspection_state as istate
@@ -29,7 +33,7 @@ from ironic_inspector import main
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
 from ironic_inspector.plugins import example as example_plugin
-from ironic_inspector import process
+from ironic_inspector.plugins import introspection_data as intros_data_plugin
 from ironic_inspector import rules
 from ironic_inspector.test import base as test_base
 from ironic_inspector import utils
@@ -42,36 +46,43 @@ def _get_error(res):
 
 
 class BaseAPITest(test_base.BaseTest):
+
+    def init_app(self):
+        CONF.set_override('auth_strategy', 'noauth')
+        main._app.testing = True
+        self.app = main.get_app().test_client()
+        self.headers = {}
+
     def setUp(self):
         super(BaseAPITest, self).setUp()
-        main.app.config['TESTING'] = True
-        self.app = main.app.test_client()
-        CONF.set_override('auth_strategy', 'noauth')
+        self.init_app()
         self.uuid = uuidutils.generate_uuid()
-
-
-class TestApiIntrospect(BaseAPITest):
-    def setUp(self):
-        super(TestApiIntrospect, self).setUp()
         self.rpc_get_client_mock = self.useFixture(
             fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
         self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
         self.rpc_get_client_mock.return_value = self.client_mock
 
-    def test_introspect_no_authentication(self):
-        CONF.set_override('auth_strategy', 'noauth')
 
-        res = self.app.post('/v1/introspection/%s' % self.uuid)
+class TestApiIntrospect(BaseAPITest):
+
+    def test_introspect(self):
+
+        res = self.app.post('/v1/introspection/%s' % self.uuid,
+                            headers=self.headers)
 
         self.assertEqual(202, res.status_code)
         self.client_mock.call.assert_called_once_with({}, 'do_introspection',
                                                       node_id=self.uuid,
                                                       manage_boot=True,
                                                       token=None)
+        self.assertEqual('application/json',
+                         res.headers['content-type'])
+        self.assertEqual(b'{}\n', res.data)
 
     def test_intospect_failed(self):
         self.client_mock.call.side_effect = utils.Error("boom")
-        res = self.app.post('/v1/introspection/%s' % self.uuid)
+        res = self.app.post('/v1/introspection/%s' % self.uuid,
+                            headers=self.headers)
 
         self.assertEqual(400, res.status_code)
         self.assertEqual(
@@ -83,16 +94,21 @@ class TestApiIntrospect(BaseAPITest):
                                                       token=None)
 
     def test_introspect_no_manage_boot(self):
-        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid)
+        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid,
+                            headers=self.headers)
         self.assertEqual(202, res.status_code)
         self.client_mock.call.assert_called_once_with({}, 'do_introspection',
                                                       node_id=self.uuid,
                                                       manage_boot=False,
                                                       token=None)
+        self.assertEqual('application/json',
+                         res.headers['content-type'])
+        self.assertEqual(b'{}\n', res.data)
 
     def test_introspect_can_manage_boot_false(self):
         CONF.set_override('can_manage_boot', False)
-        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid)
+        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid,
+                            headers=self.headers)
         self.assertEqual(202, res.status_code)
         self.client_mock.call.assert_called_once_with({}, 'do_introspection',
                                                       node_id=self.uuid,
@@ -101,12 +117,14 @@ class TestApiIntrospect(BaseAPITest):
 
     def test_introspect_can_manage_boot_false_failed(self):
         CONF.set_override('can_manage_boot', False)
-        res = self.app.post('/v1/introspection/%s' % self.uuid)
+        res = self.app.post('/v1/introspection/%s' % self.uuid,
+                            headers=self.headers)
         self.assertEqual(400, res.status_code)
         self.assertFalse(self.client_mock.call.called)
 
     def test_introspect_wrong_manage_boot(self):
-        res = self.app.post('/v1/introspection/%s?manage_boot=foo' % self.uuid)
+        res = self.app.post('/v1/introspection/%s?manage_boot=foo' % self.uuid,
+                            headers=self.headers)
         self.assertEqual(400, res.status_code)
         self.assertFalse(self.client_mock.call.called)
         self.assertEqual(
@@ -123,40 +141,66 @@ class TestApiIntrospect(BaseAPITest):
         self.assertFalse(self.client_mock.call.called)
 
 
-@mock.patch.object(process, 'process', autospec=True)
+class TestBasicAuthApiIntrospect(TestApiIntrospect):
+
+    def init_app(self):
+        CONF.set_override('auth_strategy', 'http_basic')
+        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
+            f.write('myName:$2y$05$lE3eGtyj41jZwrzS87KTqe6.'
+                    'JETVCWBkc32C63UP2aYrGoYOEpbJm\n\n\n')
+            self.addCleanup(os.remove, f.name)
+            CONF.set_override('http_basic_auth_user_file', f.name)
+        main._app.config['TESTING'] = True
+        self.app = main.get_app().test_client()
+
+        # base64 encode myName:myPassword
+        self.headers = {'Authorization': 'Basic bXlOYW1lOm15UGFzc3dvcmQ='}
+
+    def test_introspect_failed_authentication(self):
+        # base64 encode myName:yourPassword
+        self.headers = {'Authorization': 'Basic bXlOYW1lOnlvdXJQYXNzd29yZA=='}
+
+        res = self.app.post('/v1/introspection/%s' % self.uuid,
+                            headers=self.headers)
+        self.assertEqual(401, res.status_code)
+
+    def test_unauthenticated_public_api(self):
+        res = self.app.get('/')
+        self.assertEqual(200, res.status_code)
+        res = self.app.get('/v1')
+        self.assertEqual(200, res.status_code)
+        res = self.app.get('/v1/introspection')
+        self.assertEqual(401, res.status_code)
+
+
 class TestApiContinue(BaseAPITest):
-    def test_continue(self, process_mock):
+    def test_continue(self):
         # should be ignored
         CONF.set_override('auth_strategy', 'keystone')
-        process_mock.return_value = {'result': 42}
+        self.client_mock.call.return_value = {'result': 42}
         res = self.app.post('/v1/continue', data='{"foo": "bar"}')
         self.assertEqual(200, res.status_code)
-        process_mock.assert_called_once_with({"foo": "bar"})
+        self.client_mock.call.assert_called_once_with({}, 'do_continue',
+                                                      data={"foo": "bar"})
         self.assertEqual({"result": 42}, json.loads(res.data.decode()))
 
-    def test_continue_failed(self, process_mock):
-        process_mock.side_effect = utils.Error("boom")
+    def test_continue_failed(self):
+        self.client_mock.call.side_effect = utils.Error("boom")
         res = self.app.post('/v1/continue', data='{"foo": "bar"}')
         self.assertEqual(400, res.status_code)
-        process_mock.assert_called_once_with({"foo": "bar"})
+        self.client_mock.call.assert_called_once_with({}, 'do_continue',
+                                                      data={"foo": "bar"})
         self.assertEqual('boom', _get_error(res))
 
-    def test_continue_wrong_type(self, process_mock):
+    def test_continue_wrong_type(self):
         res = self.app.post('/v1/continue', data='42')
         self.assertEqual(400, res.status_code)
         self.assertEqual('Invalid data: expected a JSON object, got int',
                          _get_error(res))
-        self.assertFalse(process_mock.called)
+        self.client_mock.call.assert_not_called()
 
 
 class TestApiAbort(BaseAPITest):
-    def setUp(self):
-        super(TestApiAbort, self).setUp()
-        self.rpc_get_client_mock = self.useFixture(
-            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
-        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
-        self.rpc_get_client_mock.return_value = self.client_mock
-
     def test_ok(self):
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid,
@@ -166,7 +210,9 @@ class TestApiAbort(BaseAPITest):
                                                       node_id=self.uuid,
                                                       token='token')
         self.assertEqual(202, res.status_code)
-        self.assertEqual(b'', res.data)
+        self.assertEqual(b'{}\n', res.data)
+        self.assertEqual('application/json',
+                         res.headers['content-type'])
 
     def test_no_authentication(self):
 
@@ -176,7 +222,9 @@ class TestApiAbort(BaseAPITest):
                                                       node_id=self.uuid,
                                                       token=None)
         self.assertEqual(202, res.status_code)
-        self.assertEqual(b'', res.data)
+        self.assertEqual(b'{}\n', res.data)
+        self.assertEqual('application/json',
+                         res.headers['content-type'])
 
     def test_node_not_found(self):
         exc = utils.Error("Not Found.", code=404)
@@ -297,10 +345,9 @@ class TestApiListStatus(GetStatusAPIBaseTest):
 
 
 class TestApiGetData(BaseAPITest):
-    @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
-    def test_get_introspection_data(self, swift_mock):
-        CONF.set_override('store_data', 'swift', 'processing')
-        data = {
+    def setUp(self):
+        super(TestApiGetData, self).setUp()
+        self.introspection_data = {
             'ipmi_address': '1.2.3.4',
             'cpus': 2,
             'cpu_arch': 'x86_64',
@@ -310,44 +357,66 @@ class TestApiGetData(BaseAPITest):
                 'em1': {'mac': '11:22:33:44:55:66', 'ip': '1.2.0.1'},
             }
         }
+
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_get_introspection_data_from_swift(self, swift_mock):
+        CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
-        swift_conn.get_object.return_value = json.dumps(data)
+        swift_conn.get_object.return_value = json.dumps(
+            self.introspection_data)
         res = self.app.get('/v1/introspection/%s/data' % self.uuid)
         name = 'inspector_data-%s' % self.uuid
         swift_conn.get_object.assert_called_once_with(name)
         self.assertEqual(200, res.status_code)
-        self.assertEqual(data, json.loads(res.data.decode('utf-8')))
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
 
-    @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
-    def test_introspection_data_not_stored(self, swift_mock):
-        CONF.set_override('store_data', 'none', 'processing')
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_get_unprocessed_data_from_swift(self, swift_mock):
+        CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
+        swift_conn.get_object.return_value = json.dumps(
+            self.introspection_data)
+        res = self.app.get('/v1/introspection/%s/data/unprocessed' % self.uuid)
+        name = 'inspector_data-%s-UNPROCESSED' % self.uuid
+        swift_conn.get_object.assert_called_once_with(name)
+        self.assertEqual(200, res.status_code)
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
+
+    @mock.patch.object(intros_data_plugin, 'DatabaseStore',
+                       autospec=True)
+    def test_get_introspection_data_from_db(self, db_mock):
+        CONF.set_override('store_data', 'database', 'processing')
+        db_store = db_mock.return_value
+        db_store.get.return_value = json.dumps(self.introspection_data)
         res = self.app.get('/v1/introspection/%s/data' % self.uuid)
-        self.assertFalse(swift_conn.get_object.called)
+        db_store.get.assert_called_once_with(self.uuid, processed=True,
+                                             get_json=False)
+        self.assertEqual(200, res.status_code)
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
+
+    def test_introspection_data_not_stored(self):
+        CONF.set_override('store_data', 'none', 'processing')
+        res = self.app.get('/v1/introspection/%s/data' % self.uuid)
+        self.assertEqual(404, res.status_code)
+
+    def test_unprocessed_data_not_stored(self):
+        CONF.set_override('store_data', 'none', 'processing')
+        res = self.app.get('/v1/introspection/%s/data/unprocessed' % self.uuid)
         self.assertEqual(404, res.status_code)
 
     @mock.patch.object(ir_utils, 'get_node', autospec=True)
-    @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
-    def test_with_name(self, swift_mock, get_mock):
+    @mock.patch.object(main.process, 'get_introspection_data', autospec=True)
+    def test_with_name(self, process_mock, get_mock):
         get_mock.return_value = mock.Mock(uuid=self.uuid)
         CONF.set_override('store_data', 'swift', 'processing')
-        data = {
-            'ipmi_address': '1.2.3.4',
-            'cpus': 2,
-            'cpu_arch': 'x86_64',
-            'memory_mb': 1024,
-            'local_gb': 20,
-            'interfaces': {
-                'em1': {'mac': '11:22:33:44:55:66', 'ip': '1.2.0.1'},
-            }
-        }
-        swift_conn = swift_mock.return_value
-        swift_conn.get_object.return_value = json.dumps(data)
+        process_mock.return_value = json.dumps(self.introspection_data)
         res = self.app.get('/v1/introspection/name1/data')
-        name = 'inspector_data-%s' % self.uuid
-        swift_conn.get_object.assert_called_once_with(name)
         self.assertEqual(200, res.status_code)
-        self.assertEqual(data, json.loads(res.data.decode('utf-8')))
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
         get_mock.assert_called_once_with('name1', fields=['uuid'])
 
 
@@ -355,40 +424,45 @@ class TestApiReapply(BaseAPITest):
 
     def setUp(self):
         super(TestApiReapply, self).setUp()
-        self.rpc_get_client_mock = self.useFixture(
-            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
-        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
-        self.rpc_get_client_mock.return_value = self.client_mock
         CONF.set_override('store_data', 'swift', 'processing')
 
-    def test_ok(self):
-
+    def test_api_ok(self):
         self.app.post('/v1/introspection/%s/data/unprocessed' %
                       self.uuid)
         self.client_mock.call.assert_called_once_with({}, 'do_reapply',
-                                                      node_id=self.uuid)
+                                                      node_uuid=self.uuid,
+                                                      data=None)
 
     def test_user_data(self):
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid, data='some data')
         self.assertEqual(400, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual('User data processing is not supported yet',
-                         message)
+        self.assertIn('Invalid data: expected a JSON object', message)
         self.assertFalse(self.client_mock.call.called)
 
-    def test_swift_disabled(self):
-        CONF.set_override('store_data', 'none', 'processing')
+    def test_user_data_valid(self):
+        data = {"foo": "bar"}
+        res = self.app.post('/v1/introspection/%s/data/unprocessed' %
+                            self.uuid, data=json.dumps(data))
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=data)
+
+    def test_get_introspection_data_error(self):
+        exc = utils.Error('The store is crashed', code=404)
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid)
-        self.assertEqual(400, res.status_code)
+
+        self.assertEqual(404, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual('Inspector is not configured to store '
-                         'data. Set the [processing] store_data '
-                         'configuration option to change this.',
-                         message)
-        self.assertFalse(self.client_mock.call.called)
+        self.assertEqual(str(exc), message)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=None)
 
     def test_generic_error(self):
         exc = utils.Error('Oops', code=400)
@@ -401,11 +475,22 @@ class TestApiReapply(BaseAPITest):
         message = json.loads(res.data.decode())['error']['message']
         self.assertEqual(str(exc), message)
         self.client_mock.call.assert_called_once_with({}, 'do_reapply',
-                                                      node_id=self.uuid)
+                                                      node_uuid=self.uuid,
+                                                      data=None)
+
+    @mock.patch.object(ir_utils, 'get_node', autospec=True)
+    def test_reapply_with_node_name(self, get_mock):
+        get_mock.return_value = mock.Mock(id=self.uuid)
+        self.app.post('/v1/introspection/%s/data/unprocessed' %
+                      'fake-node')
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=None)
+        get_mock.assert_called_once_with('fake-node', fields=['uuid'])
 
 
 class TestApiRules(BaseAPITest):
-    @mock.patch.object(rules, 'get_all')
+    @mock.patch.object(rules, 'get_all', autospec=True)
     def test_get_all(self, get_all_mock):
         get_all_mock.return_value = [
             mock.Mock(spec=rules.IntrospectionRule,
@@ -432,7 +517,7 @@ class TestApiRules(BaseAPITest):
         for m in get_all_mock.return_value:
             m.as_dict.assert_called_with(short=True)
 
-    @mock.patch.object(rules, 'delete_all')
+    @mock.patch.object(rules, 'delete_all', autospec=True)
     def test_delete_all(self, delete_all_mock):
         res = self.app.delete('/v1/rules')
         self.assertEqual(204, res.status_code)
@@ -453,7 +538,8 @@ class TestApiRules(BaseAPITest):
         create_mock.assert_called_once_with(conditions_json='cond',
                                             actions_json='act',
                                             uuid=self.uuid,
-                                            description=None)
+                                            description=None,
+                                            scope=None)
         self.assertEqual(exp, json.loads(res.data.decode('utf-8')))
 
     @mock.patch.object(rules, 'create', autospec=True)
@@ -475,7 +561,8 @@ class TestApiRules(BaseAPITest):
         create_mock.assert_called_once_with(conditions_json='cond',
                                             actions_json='act',
                                             uuid=self.uuid,
-                                            description=None)
+                                            description=None,
+                                            scope=None)
         self.assertEqual(exp, json.loads(res.data.decode('utf-8')))
 
     @mock.patch.object(rules, 'create', autospec=True)
@@ -487,7 +574,7 @@ class TestApiRules(BaseAPITest):
         res = self.app.post('/v1/rules', data=json.dumps(data))
         self.assertEqual(400, res.status_code)
 
-    @mock.patch.object(rules, 'get')
+    @mock.patch.object(rules, 'get', autospec=True)
     def test_get_one(self, get_mock):
         get_mock.return_value = mock.Mock(spec=rules.IntrospectionRule,
                                           **{'as_dict.return_value':
@@ -503,11 +590,13 @@ class TestApiRules(BaseAPITest):
         get_mock.assert_called_once_with(self.uuid)
         get_mock.return_value.as_dict.assert_called_once_with(short=False)
 
-    @mock.patch.object(rules, 'delete')
+    @mock.patch.object(rules, 'delete', autospec=True)
     def test_delete_one(self, delete_mock):
         res = self.app.delete('/v1/rules/' + self.uuid)
         self.assertEqual(204, res.status_code)
         delete_mock.assert_called_once_with(self.uuid)
+        self.assertEqual('text/plain; charset=utf-8',
+                         res.headers['content-type'])
 
 
 class TestApiMisc(BaseAPITest):
@@ -565,11 +654,11 @@ class TestApiVersions(BaseAPITest):
         }]}
         self.assertEqual(expected, json_data)
 
-    @mock.patch.object(main.app.url_map, "iter_rules", autospec=True)
+    @mock.patch.object(main._app.url_map, "iter_rules", autospec=True)
     def test_version_endpoint(self, mock_rules):
         mock_rules.return_value = ["/v1/endpoint1", "/v1/endpoint2/<uuid>",
-                                   "/v1/endpoint1/<name>",
-                                   "/v2/endpoint1", "/v1/endpoint3",
+                                   "/v1/endpoint1/<name>/",
+                                   "/v2/endpoint1", "/v1/endpoint3/",
                                    "/v1/endpoint2/<uuid>/subpoint"]
         endpoint = "/v1"
         res = self.app.get(endpoint)
@@ -592,6 +681,12 @@ class TestApiVersions(BaseAPITest):
         ]}
         self.assertEqual(expected, json_data)
 
+    def test_version_endpoint_with_slash(self):
+        endpoint = "/v1/"
+        res = self.app.get(endpoint)
+        self.assertEqual(200, res.status_code)
+        self._check_version_present(res)
+
     def test_version_endpoint_invalid(self):
         endpoint = "/v-1"
         res = self.app.get(endpoint)
@@ -601,11 +696,8 @@ class TestApiVersions(BaseAPITest):
         # API version on unknown pages
         self._check_version_present(self.app.get('/v1/foobar'))
 
-    @mock.patch.object(rpc, 'get_client', autospec=True)
     @mock.patch.object(node_cache, 'get_node', autospec=True)
-    def test_usual_requests(self, get_mock, rpc_mock):
-        client_mock = mock.MagicMock(spec=messaging.RPCClient)
-        rpc_mock.return_value = client_mock
+    def test_usual_requests(self, get_mock):
         get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
                                                     started_at=42.0)
         # Successful
@@ -633,6 +725,12 @@ class TestApiVersions(BaseAPITest):
         self.assertIn('%d.%d' % main.MINIMUM_API_VERSION, error)
         self.assertIn('%d.%d' % main.CURRENT_API_VERSION, error)
 
+    def test_request_latest_version(self):
+        headers = {conf_opts.VERSION_HEADER: 'latest'}
+        res = self.app.get('/', headers=headers)
+        self.assertEqual(200, res.status_code)
+        self._check_version_present(res)
+
 
 class TestPlugins(unittest.TestCase):
     @mock.patch.object(example_plugin.ExampleProcessingHook,
@@ -640,7 +738,6 @@ class TestPlugins(unittest.TestCase):
     @mock.patch.object(example_plugin.ExampleProcessingHook,
                        'before_update', autospec=True)
     def test_hook(self, mock_post, mock_pre):
-        plugins_base._HOOKS_MGR = None
         CONF.set_override('processing_hooks', 'example', 'processing')
         mgr = plugins_base.processing_hooks_manager()
         mgr.map_method('before_processing', 'introspection_data')
@@ -651,3 +748,71 @@ class TestPlugins(unittest.TestCase):
     def test_manager_is_cached(self):
         self.assertIs(plugins_base.processing_hooks_manager(),
                       plugins_base.processing_hooks_manager())
+
+
+class TestTopic(test_base.BaseTest):
+    def setUp(self):
+        super(TestTopic, self).setUp()
+        self.transport_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'TRANSPORT',
+                                     autospec=True)).mock
+        self.target_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc.messaging, 'Target',
+                                     autospec=True)).mock
+        self.rpcclient_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc.messaging, 'RPCClient',
+                                     autospec=True)).mock
+        CONF.set_override('host', 'a-host')
+
+    def test_get_client_compat_standalone(self):
+        main.get_client_compat()
+        self.target_mock.assert_called_with(topic='ironic_inspector.conductor',
+                                            server='a-host', version=mock.ANY)
+
+    @mock.patch.object(main, 'get_random_topic', autospec=True)
+    def test_get_client_compat_non_standalone(self, mock_get_topic):
+        CONF.set_override('host', 'a-host')
+        CONF.set_override('standalone', False)
+        mock_get_topic.return_value = 'hello'
+        main.get_client_compat()
+        self.target_mock.assert_called_with(
+            topic='hello', version=mock.ANY)
+
+    @mock.patch.object(coordination, 'get_coordinator', autospec=True)
+    def test_get_random_topic(self, mock_get_coordinator):
+        mock_coordinator = mock.Mock(spec=['get_members'])
+        members = [('ironic_inspector.conductor.host%s' % i).encode('ascii')
+                   for i in range(5)]
+        topics = [('ironic_inspector.conductor.host%s' % i) for i in range(5)]
+        mock_coordinator.get_members.return_value = set(members)
+        mock_get_coordinator.return_value = mock_coordinator
+        for i in range(10):
+            topic = main.get_random_topic()
+            self.assertIn(topic, topics)
+
+    @mock.patch.object(coordination, 'get_coordinator', autospec=True)
+    def test_get_random_topic_host_with_domain(self, mock_get_coordinator):
+        mock_coordinator = mock.Mock(spec=['get_members'])
+        members = ['ironic_inspector.conductor.'
+                   'local.domain'.encode('ascii')]
+        mock_coordinator.get_members.return_value = set(members)
+        mock_get_coordinator.return_value = mock_coordinator
+        topic = main.get_random_topic()
+        self.assertEqual(topic, 'ironic_inspector.conductor.local.domain')
+
+    @mock.patch.object(coordination, 'get_coordinator', autospec=True)
+    def test_get_random_topic_host_bypass_invalid(self, mock_get_coordinator):
+        mock_coordinator = mock.Mock(spec=['get_members'])
+        members = ['this_should_not_happen'.encode('ascii')]
+        mock_coordinator.get_members.return_value = set(members)
+        mock_get_coordinator.return_value = mock_coordinator
+        self.assertRaisesRegex(utils.NoAvailableConductor,
+                               'No available conductor',
+                               main.get_random_topic)
+
+    @mock.patch.object(coordination, 'get_coordinator', autospec=True)
+    def test_get_random_topic_no_member(self, mock_get_coordinator):
+        mock_coordinator = mock.Mock(spec=['get_members'])
+        mock_coordinator.get_members.return_value = set()
+        mock_get_coordinator.return_value = mock_coordinator
+        self.assertRaises(utils.NoAvailableConductor, main.get_random_topic)
